@@ -2,7 +2,8 @@
 # bts.rb - ruby interface for debian bts
 # Copyright (C) 2002       Masato Taruishi <taru@debian.org>
 # Copyright (C) 2006-2007  Junichi Uekawa <dancer@debian.org>
-# Copyright (C) 2013-2022  Francesco Poli <invernomuto@paranoici.org>
+# Copyright (C) 2013-2023  Francesco Poli <invernomuto@paranoici.org>
+# Copyright (C) 2015       Michael Gold <michael@bitplane.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,23 +32,23 @@ module Debian
   module BTS
     class Parser
 
-      def initialize(host, port)
-        @host = host
-        @port = port
+      def initialize(url, debugfile)
+        @soapurl = url
+        @debugfile = debugfile
       end
 
       # use SOAP interface to obtain the index.
       class SoapIndex < Parser
-        def initialize(host, port)
-          @host = host
-          @port = port
+        def initialize(url, debugfile)
+          @soapurl = url
+          @debugfile = debugfile
           @indexes = {}
           @buf = nil
         end
 
         def parse_bug(bugnum)
           require 'aptlistbugs/debian/btssoap'
-          soap = Debian::BTSSOAP::Soap.new(@host, @port)
+          soap = Debian::BTSSOAP::Soap.new(@soapurl, @debugfile)
           sa = Debian::BTSSOAP::StringArray.new
 
           # query the BTS about the given bug number
@@ -60,7 +61,7 @@ module Debian
 
         def parse(ma_copies, querystep, parsestep, severities = ["critical", "grave"])
           require 'aptlistbugs/debian/btssoap'
-          soap = Debian::BTSSOAP::Soap.new(@host, @port)
+          soap = Debian::BTSSOAP::Soap.new(@soapurl, @debugfile)
           sa = Debian::BTSSOAP::StringArray.new
           bugs = Debian::Bugs.new
           reqbugs = []
