@@ -5,7 +5,7 @@
 # Copyright (C) 2002       Masato Taruishi <taru@debian.org>
 # Copyright (C) 2006-2008  Junichi Uekawa <dancer@debian.org>
 # Copyright (C) 2007       Famelis George <famelis@otenet.gr>
-# Copyright (C) 2008-2023  Francesco Poli <invernomuto@paranoici.org>
+# Copyright (C) 2008-2024  Francesco Poli <invernomuto@paranoici.org>
 # Copyright (C) 2009-2010  Ryan Niebur <ryan@debian.org>
 # Copyright (C) 2013       Google Inc
 # Copyright (C) 2015       Michael Gold <michael@bitplane.org>
@@ -57,6 +57,7 @@ class AppConfig
       _(" -T <tags>        : Filter bugs by tags you want to see.\n"),
       sprintf(_(" -S <states>      : Filter bugs by pending-state categories you want to see\n                    [%s].\n"), @stats.join(',')),
       _(" -B <bug#>        : Filter bugs by number, showing only the specified bugs.\n"),
+      _(" -r <release>     : Filter bugs by distribution release, showing only the bugs\n                    that affect the specified release.\n"),
       _(" -D               : Show downgraded packages, too.\n"),
       sprintf(_(" -u <url>         : SOAP URL for Debian Bug Tracking System\n                    [%s].\n"), @soapurl),
       _(" -H <hostname>    : Hostname of Debian Bug Tracking System\n                    (for http, deprecated).\n"),
@@ -93,6 +94,8 @@ class AppConfig
                 ["absent", _("From other Branch")],
                 ["done", _("Resolved in some Version")]]
     @fbugs = nil
+    @distro = nil
+    @other_releases = nil
     @show_downgrade = false
     @soapurl = "https://bugs.debian.org:443/cgi-bin/soap.cgi"
     @hostname = "bugs.debian.org"
@@ -115,6 +118,8 @@ class AppConfig
       # act as if root privileges could not be dropped
       @ext_user = "root"
     end
+    @ext_uid = "unknown uid"
+    @parentcmd = Process.ppid
 
     @ignore_bugs = read_ignore_bugs("/etc/apt/listbugs/ignore_bugs")
     @system_ignore_bugs = read_ignore_bugs("/var/lib/apt-listbugs/ignore_bugs")
@@ -142,7 +147,8 @@ class AppConfig
   attr_accessor :frontend, :pin_priority, :yes, :ignore_regexp, :force_pin
   attr_accessor :default_reply
   attr_reader :command, :parser, :querybts, :ignore_bugs, :system_ignore_bugs
-  attr_reader :browser, :arrow, :cmd_prefix, :ext_user, :debugfile
+  attr_reader :browser, :arrow, :cmd_prefix, :ext_user, :ext_uid, :debugfile
+  attr_reader :parentcmd, :distro, :other_releases
 
   def parse_options
     all_severities = ["critical","grave","serious","important","normal","minor","wishlist"]
@@ -154,6 +160,10 @@ class AppConfig
       end
     end
 
+    if /dr='(.*)'/ =~ `apt-config #{@apt_conf} shell dr AptListbugs::DistroRelease`
+      @distro = $1
+    end
+
     if /soap_url='(.*)'/ =~ `apt-config #{@apt_conf} shell soap_url AptListbugs::URL`
       @soapurl = $1
     end
@@ -173,6 +183,7 @@ class AppConfig
                            ['--tag', '-T', GetoptLong::REQUIRED_ARGUMENT],
                            ['--stats', '-S', GetoptLong::REQUIRED_ARGUMENT],
                            ['--bugs', '-B', GetoptLong::REQUIRED_ARGUMENT],
+                           ['--distro-release', '-r', GetoptLong::REQUIRED_ARGUMENT],
                            ['--show-downgrade', '-D', GetoptLong::NO_ARGUMENT],
                            ['--url', '-u', GetoptLong::REQUIRED_ARGUMENT],
                            ['--hostname', '-H', GetoptLong::REQUIRED_ARGUMENT],
@@ -211,6 +222,8 @@ class AppConfig
           @stats = optargs.split(',')
         when '--bugs'
           @fbugs = optargs.split(',')
+        when '--distro-release'
+          @distro = optargs
         when '--show-downgrade'
           @show_downgrade = true
         when '--url'
@@ -282,6 +295,39 @@ class AppConfig
 
     @title = sprintf(_("Bugs of severity %s"), @severity.join(', ')) if ! @title
 
+    @distro = nil if @distro == "ANY"
+    if ! @distro.nil?
+      case @distro
+      when "unstable"
+        o, e, s = Open3.capture3("distro-info --codename --devel")
+        if s.success?
+          @distro = o.chomp
+        end
+      when "testing", "stable", "oldstable"
+        o, e, s = Open3.capture3("distro-info --codename --#{@distro}")
+        if s.success?
+          @distro = o.chomp
+        end
+      end
+
+      # abort on unknown (possibly misspelled) distribution release
+      c = "distro-info --codename --all"
+      o, e, s = Open3.capture3(c)
+      if s.success?
+        @other_releases = o.split("\n")
+        if @other_releases.delete(@distro).nil?
+          $stderr.puts _("E: ") + sprintf(_("Unrecognized distribution release '%s'."), @distro)
+          exit 1
+        end
+      else
+        $stderr.puts _("E: ") + sprintf(_("Cannot determine list of distribution releases with command %s"), c)
+        exit 1
+      end
+
+      @debugfile.puts "Distribution release: #{@distro}" if $DEBUG
+      @debugfile.puts "Other known releases: #{@other_releases.join(",")}" if $DEBUG
+    end
+
     # http_proxy sanity check
     if ENV["HTTP_PROXY"] != nil && ENV["http_proxy"] == nil
       $stderr.puts _("W: ") + _("sanity check failed: environment variable http_proxy is unset and HTTP_PROXY is set.")
@@ -385,6 +431,19 @@ class AppConfig
       end
       # if the effective user name cannot be determined, leave it empty
     end
+    o, e, s = Open3.capture3("id -u #{@ext_user}")
+    if s.success? && o != nil
+      @ext_uid = o.chomp
+    end
+
+    @debugfile.puts "PPID is #{@parentcmd}" if $DEBUG
+    begin
+      require 'sys/proctable'
+      @parentcmd = Sys::ProcTable.ps(pid: @parentcmd).ppid
+      @parentcmd = Sys::ProcTable.ps(pid: @parentcmd).cmdline
+    rescue LoadError
+      @parentcmd = nil
+    end
 
     if /ignore_regexp='(.*)'/ =~ `apt-config #{@apt_conf} shell ignore_regexp AptListbugs::IgnoreRegexp`
       @ignore_regexp = $1
@@ -801,6 +860,8 @@ class Viewer
           end
           pinned_pkg_keys << pkg_key
           holdstr << "\nExplanation: Pinned by apt-listbugs at #{Time.now}"
+          holdstr << "\nExplanation: Pinning requested by #{@config.ext_user} (#{@config.ext_uid})"
+          holdstr << " during command: #{@config.parentcmd}" if ! @config.parentcmd.nil?
           my_bugs.each { |bug|
             holdstr << "\nExplanation:   ##{bug.bug_number}: #{bug.desc}"
             explanation_bugs << bug.bug_number
@@ -812,9 +873,12 @@ class Viewer
       @config.debugfile.puts holdstr if $DEBUG
       if holdstr != ""
         @pref_str << holdstr
-        # TRANSLATORS: %{packgl} is a list of packages.
-        @config.frontend.puts _("%{packgl} will be pinned. Restart APT session to enable") %
-              {:packgl => pinned_pkg_keys.join(' ')}
+        # TRANSLATORS: %{packgl} is a comma-separated list of %{npackg} packages.
+        @config.frontend.puts ngettext(
+         "Restart APT session to enable pinning for the following %{npackg} package:\n %{packgl}\n",
+         "Restart APT session to enable pinning for the following %{npackg} packages:\n %{packgl}\n",
+         pinned_pkg_keys.size) % {:npackg => pinned_pkg_keys.size,
+                                  :packgl => pinned_pkg_keys.join(', ')}
         return pinned_pkg_keys, explanation_bugs
       end
       return nil
@@ -1360,7 +1424,7 @@ module Factory
         config.debugfile.puts "#{bug}" if $DEBUG
         config.tag.each { |tag|
           if bug.tags && bug.tags.include?( tag )
-            config.debugfile.puts "#{bug} has {tag}" if $DEBUG
+            config.debugfile.puts "#{bug} is tagged #{tag}" if $DEBUG
           else
             val = true
           end
@@ -1369,12 +1433,32 @@ module Factory
       }
     end
 
+    def delete_not_for_us_bugs( bugs )
+      config.debugfile.puts "checking not-for-us bugs: #{bugs}" if $DEBUG
+      bugs.delete_if { |bug|
+        val = false
+        config.debugfile.puts "#{bug}" if $DEBUG
+        if bug.tags && ! bug.tags.include?( config.distro )
+          config.debugfile.puts "#{bug} does not specifically affect our release (#{config.distro})" if $DEBUG
+          config.other_releases.reverse_each { |release|
+            if bug.tags.include?( release )
+              config.debugfile.puts "#{bug} specifically affects release #{release}" if $DEBUG
+              val = true
+              break
+            end
+          }
+        end
+        val
+      }
+    end
+
     module_function :delete_ignore_pkgs, :create, :delete_ignore_bugs,
     :delete_uninteresting_bugs,
     :delete_regexp_bugs,
     :bug_is_irrelevant,
     :am_i_buggy,
     :delete_irrelevant_bugs, :delete_unwanted_tag_bugs,
+    :delete_not_for_us_bugs,
     :find_max_version_below_ver,
     :find_min_version_above_ver,
     :iterate_fixed_found_version
